@@ -1,4 +1,4 @@
-{-# LANGUAGE PatternGuards, DeriveDataTypeable #-}
+{-# LANGUAGE PatternGuards, DeriveDataTypeable, TupleSections #-}
 {-# OPTIONS_GHC -fno-warn-missing-fields -fno-cse -O0 #-}
 
 module CmdLine(
@@ -28,7 +28,9 @@ import System.IO
 import System.IO.Error
 import System.Info.Extra
 import System.Process
+import System.FilePattern
 
+import EmbedData
 import Util
 import Paths_hlint
 import Data.Version
@@ -122,6 +124,7 @@ data Cmd
         ,cmdRefactor :: Bool            -- ^ Run the `refactor` executable to automatically perform hints
         ,cmdRefactorOptions :: String   -- ^ Options to pass to the `refactor` executable.
         ,cmdWithRefactor :: FilePath    -- ^ Path to refactor tool
+        ,cmdIgnoreGlob :: [FilePattern]
         }
     | CmdGrep
         {cmdFiles :: [FilePath]    -- ^ which files to run it on, nothing = none given
@@ -185,6 +188,7 @@ mode = cmdArgsMode $ modes
         ,cmdRefactor = nam_ "refactor" &= help "Automatically invoke `refactor` to apply hints"
         ,cmdRefactorOptions = nam_ "refactor-options" &= typ "OPTIONS" &= help "Options to pass to the `refactor` executable"
         ,cmdWithRefactor = nam_ "with-refactor" &= help "Give the path to refactor"
+        ,cmdIgnoreGlob = nam_ "ignore-glob" &= help "Ignore paths matching glob pattern"
         } &= auto &= explicit &= name "lint"
     ,CmdGrep
         {cmdFiles = def &= args &= typ "FILE/DIR"
@@ -203,7 +207,7 @@ mode = cmdArgsMode $ modes
     ,CmdHSE
         {} &= explicit &= name "hse"
     ] &= program "hlint" &= verbosity
-    &=  summary ("HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2019")
+    &=  summary ("HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2020")
     where
         nam xs = nam_ xs &= name [head xs]
         nam_ xs = def &= explicit &= name xs
@@ -212,14 +216,15 @@ mode = cmdArgsMode $ modes
 --   * If someone passes cmdWithHints, only look at files they explicitly request
 --   * If someone passes an explicit hint name, automatically merge in data/hlint.yaml
 --   We want more important hints to go last, since they override
-cmdHintFiles :: Cmd -> IO [FilePath]
+cmdHintFiles :: Cmd -> IO [(FilePath, Maybe String)]
 cmdHintFiles cmd = do
-    let explicit1 = [cmdDataDir cmd </> "hlint.yaml" | null $ cmdWithHints cmd]
+    let explicit1 = [hlintYaml | null $ cmdWithHints cmd]
     let explicit2 = cmdGivenHints cmd
-    bad <- filterM (notM . doesFileExist) $ explicit1 ++ explicit2
+    bad <- filterM (notM . doesFileExist) explicit2
+    let explicit2' = map (,Nothing) explicit2
     when (bad /= []) $
         fail $ unlines $ "Failed to find requested hint files:" : map ("  "++) bad
-    if cmdWithHints cmd /= [] then return $ explicit1 ++ explicit2 else do
+    if cmdWithHints cmd /= [] then return $ explicit1 ++ explicit2' else do
         -- we follow the stylish-haskell config file search policy
         -- 1) current directory or its ancestors; 2) home directory
         curdir <- getCurrentDirectory
@@ -228,7 +233,7 @@ cmdHintFiles cmd = do
         implicit <- findM doesFileExist $
             map (</> ".hlint.yaml") (ancestors curdir ++ home) -- to match Stylish Haskell
             ++ ["HLint.hs"] -- the default in HLint 1.*
-        return $ explicit1 ++ maybeToList implicit ++ explicit2
+        return $ explicit1 ++ map (,Nothing) (maybeToList implicit) ++ explicit2'
     where
         ancestors = init . map joinPath . reverse . inits . splitPath
 
@@ -265,20 +270,29 @@ resolveFile
     -> Maybe FilePath -- ^ Temporary file
     -> FilePath       -- ^ File to resolve, may be "-" for stdin
     -> IO [FilePath]
-resolveFile cmd = getFile (cmdPath cmd) (cmdExtension cmd)
+resolveFile cmd = getFile (toPredicate $ cmdIgnoreGlob cmd) (cmdPath cmd) (cmdExtension cmd)
+    where
+        toPredicate :: [FilePattern] -> FilePath -> Bool
+        toPredicate [] = const False
+        toPredicate globs = \x -> not $ null $ m [((), cleanup x)]
+            where m = matchMany (map ((),) globs)
+
+        cleanup :: FilePath -> FilePath
+        cleanup ('.':x:xs) | isPathSeparator x, not $ null xs = xs
+        cleanup x = x
 
 
-getFile :: [FilePath] -> [String] -> Maybe FilePath -> FilePath -> IO [FilePath]
-getFile path _ (Just tmpfile) "-" =
+getFile :: (FilePath -> Bool) -> [FilePath] -> [String] -> Maybe FilePath -> FilePath -> IO [FilePath]
+getFile _ path _ (Just tmpfile) "-" =
     -- make sure we don't reencode any Unicode
     BS.getContents >>= BS.writeFile tmpfile >> return [tmpfile]
-getFile path _ Nothing "-" = return ["-"]
-getFile [] exts _ file = exitMessage $ "Couldn't find file: " ++ file
-getFile (p:ath) exts t file = do
+getFile _ path _ Nothing "-" = return ["-"]
+getFile _ [] exts _ file = exitMessage $ "Couldn't find file: " ++ file
+getFile ignore (p:ath) exts t file = do
     isDir <- doesDirectoryExist $ p <\> file
     if isDir then do
         let avoidDir x = let y = takeFileName x in "_" `isPrefixOf` y || ("." `isPrefixOf` y && not (all (== '.') y))
-            avoidFile x = let y = takeFileName x in "." `isPrefixOf` y
+            avoidFile x = let y = takeFileName x in "." `isPrefixOf` y || ignore x
         xs <- listFilesInside (return . not . avoidDir) $ p <\> file
         return [x | x <- xs, drop 1 (takeExtension x) `elem` exts, not $ avoidFile x]
      else do
@@ -288,7 +302,7 @@ getFile (p:ath) exts t file = do
             res <- getModule p exts file
             case res of
                 Just x -> return [x]
-                Nothing -> getFile ath exts t file
+                Nothing -> getFile ignore ath exts t file
 
 
 getModule :: FilePath -> [String] -> FilePath -> IO (Maybe FilePath)
@@ -308,7 +322,7 @@ getModule _ _ _ = return Nothing
 
 
 getExtensions :: [String] -> (Language, [Extension])
-getExtensions args = (lang, foldl f (if null langs then defaultExtensions else []) exts)
+getExtensions args = (lang, foldl f (if null langs then parseExtensions else []) exts)
     where
         lang = if null langs then baseLanguage defaultParseMode else fromJust $ lookup (last langs) ls
         (langs, exts) = partition (isJust . flip lookup ls) args
