@@ -1,6 +1,6 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE ViewPatterns #-}
-{-# LANGUAGE PackageImports #-}
+{-# LANGUAGE TupleSections #-}
 
 module HSE.All(
     module X,
@@ -9,13 +9,13 @@ module HSE.All(
     ParseError(..), ModuleEx(..),
     parseModuleEx, ghcComments,
     freeVars, vars, varss, pvars,
-    ghcSpanToHSE, ghcSrcLocToHSE
+    ghcSpanToHSE, ghcSrcLocToHSE,
+    parseExpGhcWithMode, parseImportDeclGhcWithMode
     ) where
 
 import Language.Haskell.Exts.Util hiding (freeVars, Vars(..))
 import qualified Language.Haskell.Exts.Util as X
 import HSE.Util as X
-import HSE.Reduce as X
 import HSE.Type as X
 import HSE.Match as X
 import HSE.Scope as X
@@ -33,15 +33,19 @@ import System.IO.Extra
 import Data.Functor
 import Prelude
 
+import qualified HsSyn
+import qualified FastString
+import qualified SrcLoc as GHC
+import qualified ErrUtils
+import qualified Outputable
+import qualified Lexer as GHC
+import qualified GHC.LanguageExtensions.Type as GHC
+import qualified ApiAnnotation as GHC
+import qualified BasicTypes as GHC
+import qualified DynFlags as GHC
+
 import GHC.Util
-import qualified "ghc-lib-parser" HsSyn
-import qualified "ghc-lib-parser" FastString
-import qualified "ghc-lib-parser" SrcLoc as GHC
-import qualified "ghc-lib-parser" ErrUtils
-import qualified "ghc-lib-parser" Outputable
-import qualified "ghc-lib-parser" Lexer as GHC
-import qualified "ghc-lib-parser" GHC.LanguageExtensions.Type as GHC
-import qualified "ghc-lib-parser" ApiAnnotation as GHC
+import qualified Language.Haskell.GhclibParserEx.Fixity as GhclibParserEx
 
 -- | Convert a GHC source loc into an HSE equivalent.
 ghcSrcLocToHSE :: GHC.SrcLoc -> SrcLoc
@@ -151,7 +155,7 @@ defaultParseFlags = ParseFlags NoCpp defaultParseMode
     {fixities = Just $ customFixities ++ baseFixities ++ baseNotYetInHSE ++ lensFixities ++ otherFixities
     ,ignoreLinePragmas = False
     ,ignoreFunctionArity = True
-    ,extensions = defaultExtensions}
+    ,extensions = parseExtensions}
 
 parseFlagsNoLocations :: ParseFlags -> ParseFlags
 parseFlagsNoLocations x = x{cppFlags = case cppFlags x of Cpphs y -> Cpphs $ f y; y -> y}
@@ -189,12 +193,12 @@ data ParseError = ParseError
 data ModuleEx = ModuleEx {
     hseModule :: Module SrcSpanInfo
   , hseComments :: [Comment]
-  , ghcModule :: Located (HsSyn.HsModule HsSyn.GhcPs)
+  , ghcModule :: GHC.Located (HsSyn.HsModule HsSyn.GhcPs)
   , ghcAnnotations :: GHC.ApiAnns
 }
 
 -- | Extract a list of all of a parsed module's comments.
-ghcComments :: ModuleEx -> [Located GHC.AnnotationComment]
+ghcComments :: ModuleEx -> [GHC.Located GHC.AnnotationComment]
 ghcComments m = concat (Map.elems $ snd (ghcAnnotations m))
 
 -- | Utility called from 'parseModuleEx' and 'hseFailOpParseModuleEx'.
@@ -261,11 +265,36 @@ ghcFailOpParseModuleEx ppstr file str (loc, err) = do
                ErrUtils.pprLocErrMsg (ErrUtils.mkPlainErrMsg baseDynFlags loc err)
    return $ Left $ ParseError sl msg pe
 
--- | Produce a pair of lists from a 'ParseFlags' value representing
--- language extensions to explicitly enable/disable.
-ghcExtensionsFromParseFlags :: ParseFlags
-                             -> ([GHC.Extension], [GHC.Extension])
-ghcExtensionsFromParseFlags ParseFlags {hseFlags=ParseMode {extensions=exts}}=
+-- A hacky function to get fixities from HSE parse flags suitable for
+-- use by our own 'GHC.Util.Refact.Fixity' module.
+ghcFixitiesFromParseMode :: ParseMode -> [(String, GHC.Fixity)]
+ghcFixitiesFromParseMode ParseMode {fixities=Just fixities} =
+  concatMap convert fixities
+  where
+    convert (Fixity (AssocNone _) fix name) = infix_' fix [qNameToStr name]
+    convert (Fixity (AssocLeft _) fix name) = infixl_' fix [qNameToStr name]
+    convert (Fixity (AssocRight _) fix name) = infixr_' fix [qNameToStr name]
+
+    infixr_', infixl_', infix_' :: Int -> [String] -> [(String,GHC.Fixity)]
+    infixr_' = fixity' GHC.InfixR
+    infixl_' = fixity' GHC.InfixL
+    infix_'  = fixity' GHC.InfixN
+
+    fixity' :: GHC.FixityDirection -> Int -> [String] -> [(String, GHC.Fixity)]
+    fixity' a p = map (,GHC.Fixity (GHC.SourceText "") p a)
+
+    qNameToStr :: QName () -> String
+    qNameToStr (Special _ Cons{}) = ":"
+    qNameToStr (Special _ UnitCon{}) = "()"
+    qNameToStr (UnQual _ (X.Ident _ x)) = x
+    qNameToStr (UnQual _ (Symbol _ x)) = x
+    qNameToStr _ = ""
+ghcFixitiesFromParseMode _ = []
+
+-- GHC enabled/disabled extensions given an HSE parse mode.
+ghcExtensionsFromParseMode :: ParseMode
+                           -> ([GHC.Extension], [GHC.Extension])
+ghcExtensionsFromParseMode ParseMode {extensions=exts}=
    partitionEithers $ mapMaybe toEither exts
    where
      toEither ke = case ke of
@@ -274,6 +303,33 @@ ghcExtensionsFromParseFlags ParseFlags {hseFlags=ParseMode {extensions=exts}}=
        UnknownExtension ('N':'o':e) -> Right <$> readExtension e
        UnknownExtension e -> Left <$> readExtension e
 
+-- GHC extensions to enable/disable given HSE parse flags.
+ghcExtensionsFromParseFlags :: ParseFlags
+                             -> ([GHC.Extension], [GHC.Extension])
+ghcExtensionsFromParseFlags ParseFlags {hseFlags=mode} = ghcExtensionsFromParseMode mode
+
+-- GHC fixities given HSE parse flags.
+ghcFixitiesFromParseFlags :: ParseFlags -> [(String, GHC.Fixity)]
+ghcFixitiesFromParseFlags ParseFlags {hseFlags=mode} = ghcFixitiesFromParseMode mode
+
+-- These next two functions get called frorm 'Config/Yaml.hs' for user
+-- defined hint rules.
+
+parseExpGhcWithMode :: ParseMode -> String -> GHC.ParseResult (HsSyn.LHsExpr HsSyn.GhcPs)
+parseExpGhcWithMode parseMode s =
+  let (enable, disable) = ghcExtensionsFromParseMode parseMode
+      flags = foldl' GHC.xopt_unset (foldl' GHC.xopt_set baseDynFlags enable) disable
+      fixities = ghcFixitiesFromParseMode parseMode
+  in case parseExpGhcLib s flags of
+    GHC.POk pst a -> GHC.POk pst (GhclibParserEx.applyFixities fixities a)
+    f@GHC.PFailed{} -> f
+
+parseImportDeclGhcWithMode :: ParseMode -> String -> GHC.ParseResult (HsSyn.LImportDecl HsSyn.GhcPs)
+parseImportDeclGhcWithMode parseMode s =
+  let (enable, disable) = ghcExtensionsFromParseMode parseMode
+      flags = foldl' GHC.xopt_unset (foldl' GHC.xopt_set baseDynFlags enable) disable
+  in parseImportGhcLib s flags
+
 -- | Parse a Haskell module. Applies the C pre processor, and uses
 -- best-guess fixity resolution if there are ambiguities.  The
 -- filename @-@ is treated as @stdin@. Requires some flags (often
@@ -288,19 +344,21 @@ parseModuleEx flags file str = timedIO "Parse" file $ do
         str <- return $ fromMaybe str $ stripPrefix "\65279" str -- remove the BOM if it exists, see #130
         ppstr <- runCpp (cppFlags flags) file str
         let enableDisableExts = ghcExtensionsFromParseFlags flags
+            fixities = ghcFixitiesFromParseFlags flags -- Note : Fixities are coming from HSE parse flags.
         dynFlags <- parsePragmasIntoDynFlags baseDynFlags enableDisableExts file ppstr
         case dynFlags of
           Right ghcFlags ->
             case (parseFileContentsWithComments (mkMode flags file) ppstr, parseFileGhcLib file ppstr ghcFlags) of
-                (ParseOk (x, cs), POk pst a) ->
+                (ParseOk (x, cs), GHC.POk pst a) ->
                     let anns =
                           ( Map.fromListWith (++) $ GHC.annotations pst
                           , Map.fromList ((GHC.noSrcSpan, GHC.comment_q pst) : GHC.annotations_comments pst)
                           ) in
-                    return $ Right (ModuleEx (applyFixity fixity x) cs a anns)
+                    let a' = GhclibParserEx.applyFixities fixities a in
+                    return $ Right (ModuleEx (applyFixity fixity x) cs a' anns)
                 -- Parse error if GHC parsing fails (see
                 -- https://github.com/ndmitchell/hlint/issues/645).
-                (ParseOk _, PFailed _ loc err) ->
+                (ParseOk _,  GHC.PFailed _ loc err) ->
                     ghcFailOpParseModuleEx ppstr file str (loc, err)
                 (ParseFailed sl msg, pfailed) ->
                     failOpParseModuleEx ppstr flags file str sl msg $ fromPFailed pfailed
@@ -314,7 +372,7 @@ parseModuleEx flags file str = timedIO "Parse" file $ do
             return $ Left (ParseError loc msg (context (srcLine loc) ppstr))
 
     where
-        fromPFailed (PFailed _ loc err) = Just (loc, err)
+        fromPFailed (GHC.PFailed _ loc err) = Just (loc, err)
         fromPFailed _ = Nothing
 
         fixity = fromMaybe [] $ fixities $ hseFlags flags
