@@ -395,10 +395,11 @@ extensionImpliedBy = \x -> Map.findWithDefault [] x mp
 
 -- | (a, bs) means extension a implies all of bs.
 --   Taken from https://downloads.haskell.org/~ghc/master/users-guide/glasgow_exts.html#language-options
+--   In the GHC source at DynFlags.impliedXFlags
 extensionImplications :: [(Extension, [Extension])]
 extensionImplications = map (first EnableExtension) $
     (RebindableSyntax, [DisableExtension ImplicitPrelude]) :
-    map (\(k, vs) -> (k, map EnableExtension vs))
+    map (second (map EnableExtension))
     [ (DerivingVia              , [DerivingStrategies])
     , (RecordWildCards          , [DisambiguateRecordFields])
     , (ExistentialQuantification, [ExplicitForAll])
