@@ -44,23 +44,26 @@ data HintBuiltin =
 
 builtin :: HintBuiltin -> Hint
 builtin x = case x of
-    HintList       -> decl listHint
-    HintListRec    -> decl listRecHint
-    HintMonad      -> decl monadHint
+    -- Hse.
     HintLambda     -> decl lambdaHint
-    HintBracket    -> decl bracketHint
-    HintNaming     -> decl' namingHint
-    HintPattern    -> decl patternHint
+
+    -- Ghc.
     HintImport     -> modu importHint
     HintExport     -> modu exportHint
     HintComment    -> modu commentHint
     HintPragma     -> modu pragmaHint
-    HintExtensions -> modu extensionsHint
-    HintUnsafe     -> decl unsafeHint
     HintDuplicate  -> mods duplicateHint
-    HintNewType    -> decl' newtypeHint
     HintRestrict   -> mempty{hintModule=restrictHint}
-    HintSmell      -> mempty{hintDecl=smellHint,hintModule=smellModuleHint}
+    HintList       -> decl' listHint
+    HintNewType    -> decl' newtypeHint
+    HintUnsafe     -> decl' unsafeHint
+    HintListRec    -> decl' listRecHint
+    HintNaming     -> decl' namingHint
+    HintBracket    -> decl' bracketHint
+    HintSmell      -> mempty{hintDecl'=smellHint,hintModule=smellModuleHint}
+    HintPattern    -> decl' patternHint
+    HintMonad      -> decl' monadHint
+    HintExtensions -> modu extensionsHint
     where
         wrap = timed "Hint" (drop 4 $ show x) . forceList
         decl f = mempty{hintDecl=const $ \a b c -> wrap $ f a b c}
@@ -74,8 +77,9 @@ builtinHints = [(drop 4 $ show h, builtin h) | h <- [minBound .. maxBound]]
 
 -- | Transform a list of 'HintBuiltin' or 'HintRule' into a 'Hint'.
 resolveHints :: [Either HintBuiltin HintRule] -> Hint
-resolveHints xs = mconcat $ mempty{hintDecl=const $ readMatch rights} : map builtin (nubOrd lefts)
-    where (lefts,rights) = partitionEithers xs
+resolveHints xs =
+  mconcat $ mempty{hintDecl'=const $ readMatch' rights} : map builtin (nubOrd lefts)
+  where (lefts,rights) = partitionEithers xs
 
 -- | Transform a list of 'HintRule' into a 'Hint'.
 hintRules :: [HintRule] -> Hint
