@@ -1,10 +1,9 @@
 {-# LANGUAGE RecordWildCards, NoMonomorphismRestriction #-}
-{-# LANGUAGE PackageImports #-}
 
 module Idea(
     Idea(..),
-    rawIdea, idea, idea', suggest, suggest', warn, warn',ignore, ignore',
-    rawIdeaN, suggestN, suggestN', ignoreN, ignoreNoSuggestion',
+    rawIdea, rawIdea', idea, idea', suggest, suggest', warn, warn',ignore, ignore',
+    rawIdeaN, rawIdeaN', suggestN, suggestN', ignoreN, ignoreN', ignoreNoSuggestion',
     showIdeasJson, showANSI,
     Note(..), showNotes,
     Severity(..),
@@ -18,8 +17,8 @@ import HsColour
 import Refact.Types hiding (SrcSpan)
 import qualified Refact.Types as R
 import Prelude
-import qualified "ghc-lib-parser" SrcLoc as GHC
-import qualified "ghc-lib-parser" Outputable
+import qualified SrcLoc as GHC
+import qualified Outputable
 import qualified GHC.Util as GHC
 
 -- | An idea suggest by a 'Hint'.
@@ -84,16 +83,23 @@ showEx tt Idea{..} = unlines $
 
 rawIdea :: Severity -> String -> SrcSpan -> String -> Maybe String -> [Note]-> [Refactoring R.SrcSpan] -> Idea
 rawIdea = Idea [] []
+
+rawIdea' :: Severity -> String -> GHC.SrcSpan -> String -> Maybe String -> [Note]-> [Refactoring R.SrcSpan] -> Idea
+rawIdea' a b c = Idea [] [] a b (ghcSpanToHSE c)
+
 rawIdeaN :: Severity -> String -> SrcSpan -> String -> Maybe String -> [Note] -> Idea
 rawIdeaN a b c d e f = Idea [] [] a b c d e f []
 
+rawIdeaN' :: Severity -> String -> GHC.SrcSpan -> String -> Maybe String -> [Note] -> Idea
+rawIdeaN' a b c d e f = Idea [] [] a b (ghcSpanToHSE c) d e f []
+
 idea :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
         Severity -> String -> ast SrcSpanInfo -> a -> [Refactoring R.SrcSpan] -> Idea
 idea severity hint from to = rawIdea severity hint (srcInfoSpan $ ann from) (f from) (Just $ f to) []
     where f = trimStart . prettyPrint
 
-idea' :: (GHC.HasSrcSpan a, Outputable.Outputable a) =>
-         Severity -> String -> a -> a -> [Refactoring R.SrcSpan] -> Idea
+idea' :: (GHC.HasSrcSpan a, Outputable.Outputable a, GHC.HasSrcSpan b, Outputable.Outputable b) =>
+         Severity -> String -> a -> b -> [Refactoring R.SrcSpan] -> Idea
 idea' severity hint from to =
   rawIdea severity hint (ghcSpanToHSE (GHC.getLoc from)) (GHC.unsafePrettyPrint from) (Just $ GHC.unsafePrettyPrint to) []
 
@@ -101,16 +107,16 @@ suggest :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
            String -> ast SrcSpanInfo -> a -> [Refactoring R.SrcSpan] -> Idea
 suggest = idea Suggestion
 
-suggest' :: (GHC.HasSrcSpan a, Outputable.Outputable a) =>
-            String -> a -> a -> [Refactoring R.SrcSpan] -> Idea
+suggest' :: (GHC.HasSrcSpan a, Outputable.Outputable a, GHC.HasSrcSpan b, Outputable.Outputable b) =>
+            String -> a -> b -> [Refactoring R.SrcSpan] -> Idea
 suggest' = idea' Suggestion
 
 warn :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
         String -> ast SrcSpanInfo -> a -> [Refactoring R.SrcSpan] -> Idea
 warn = idea Warning
 
-warn' :: (GHC.HasSrcSpan a, Outputable.Outputable a) =>
-         String -> a -> a -> [Refactoring R.SrcSpan] -> Idea
+warn' :: (GHC.HasSrcSpan a, Outputable.Outputable a, GHC.HasSrcSpan b, Outputable.Outputable b) =>
+         String -> a -> b -> [Refactoring R.SrcSpan] -> Idea
 warn' = idea' Warning
 
 ignoreNoSuggestion' :: (GHC.HasSrcSpan a, Outputable.Outputable a)
@@ -144,3 +150,7 @@ suggestN' = ideaN' Suggestion
 ignoreN :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
            String -> ast SrcSpanInfo -> a -> Idea
 ignoreN = ideaN Ignore
+
+ignoreN' :: (GHC.HasSrcSpan a, Outputable.Outputable a) =>
+           String -> a -> a -> Idea
+ignoreN' = ideaN' Ignore
