@@ -1,4 +1,3 @@
-{-# LANGUAGE PackageImports #-}
 module Refact
     ( toRefactSrcSpan
     , toSS, toSS'
@@ -8,7 +7,7 @@ module Refact
 import qualified Refact.Types as R
 import HSE.All
 
-import qualified "ghc-lib-parser" SrcLoc as GHC
+import qualified SrcLoc as GHC
 
 toRefactSrcSpan :: SrcSpan -> R.SrcSpan
 toRefactSrcSpan ss = R.SrcSpan (srcSpanStartLine ss)
@@ -31,5 +30,5 @@ toSrcSpan' x = case GHC.getLoc x of
     GHC.UnhelpfulSpan _ ->
         R.SrcSpan 0 0 0 0
 
-toSS' :: GHC.Located e -> R.SrcSpan
+toSS' :: GHC.HasSrcSpan e => e -> R.SrcSpan
 toSS' = toRefactSrcSpan . ghcSpanToHSE . GHC.getLoc
