@@ -5,20 +5,19 @@ module Report(writeReport) where
 import Idea
 import Data.Tuple.Extra
 import Data.List.Extra
+import qualified Data.List.NonEmpty as NE
 import Data.Maybe
 import Data.Version
-import System.FilePath
-import System.IO.Extra
 import HSE.All
 import Timing
 import Paths_hlint
 import HsColour
+import EmbedData
 
 
 writeTemplate :: FilePath -> [(String,[String])] -> FilePath -> IO ()
-writeTemplate dataDir content to = do
-    src <- readFile' $ dataDir </> "report_template.html"
-    writeFile to $ unlines $ concatMap f $ lines src
+writeTemplate dataDir content to =
+    writeFile to $ unlines $ concatMap f $ lines reportTemplate
     where
         f ('$':xs) = fromMaybe ['$':xs] $ lookup xs content
         f x = [x]
@@ -28,7 +27,7 @@ writeReport :: FilePath -> FilePath -> [Idea] -> IO ()
 writeReport dataDir file ideas = timedIO "Report" file $ writeTemplate dataDir inner file
     where
         generateIds :: [String] -> [(String,Int)] -- sorted by name
-        generateIds = map (head &&& length) . group -- must be already sorted
+        generateIds = map (NE.head &&& length) . NE.group -- must be already sorted
         files = generateIds $ sort $ map (srcSpanFilename . ideaSpan) ideas
         hints = generateIds $ map hintName $ sortOn (negate . fromEnum . ideaSeverity &&& hintName) ideas
         hintName x = show (ideaSeverity x) ++ ": " ++ ideaHint x
